import pandas as pd
import numpy as np
from sklearn.cluster import KMeans
from sklearn.decomposition import PCA
from sklearn.preprocessing import StandardScaler
import matplotlib.pyplot as plt

# 1. ダミーデータの作成（顧客データ）
np.random.seed(42)
data = {
    "Age": np.random.randint(18, 65, 200),
    "Income": np.random.randint(20000, 120000, 200),
    "Spending_Score": np.random.randint(1, 100, 200)
}
df = pd.DataFrame(data)

# 2. データの標準化
scaler = StandardScaler()
scaled_data = scaler.fit_transform(df)

# 3. k-meansクラスタリングの適用
kmeans = KMeans(n_clusters=4, random_state=42)
df['Cluster'] = kmeans.fit_predict(scaled_data)

# 4. PCAによる次元削減と可視化
pca = PCA(n_components=2)
reduced_data = pca.fit_transform(scaled_data)
df['PCA1'] = reduced_data[:, 0]
df['PCA2'] = reduced_data[:, 1]

# 可視化
plt.figure(figsize=(10, 7))
for cluster in df['Cluster'].unique():
    clustered = df[df['Cluster'] == cluster]
    plt.scatter(clustered['PCA1'], clustered['PCA2'], label=f'Cluster {cluster}')
plt.xlabel('PCA1')
plt.ylabel('PCA2')
plt.title('Customer Segmentation via K-means Clustering')
plt.legend()
plt.grid()
plt.show()

# 5. クラスタごとの特徴の分析
cluster_summary = df.groupby('Cluster')[['Age', 'Income', 'Spending_Score']].mean()
print("クラスタごとの平均特性:")
print(cluster_summary)
