import numpy as np
import pandas as pd
from sklearn.cluster import DBSCAN
import matplotlib.pyplot as plt

# 1. ダミーセンサーデータの作成
np.random.seed(42)
sensor_data = {
    "Sensor_1": np.random.normal(50, 5, 300),
    "Sensor_2": np.random.normal(60, 5, 300)
}
# Adding anomalies
anomalies = {
    "Sensor_1": np.random.uniform(80, 100, 10),
    "Sensor_2": np.random.uniform(80, 100, 10)
}
sensor_data["Sensor_1"] = np.concatenate([sensor_data["Sensor_1"], anomalies["Sensor_1"]])
sensor_data["Sensor_2"] = np.concatenate([sensor_data["Sensor_2"], anomalies["Sensor_2"]])

df = pd.DataFrame(sensor_data)

# 2. DBSCANの適用
dbscan = DBSCAN(eps=3, min_samples=5)
df['Cluster'] = dbscan.fit_predict(df)

# 3. クラスタリング結果の可視化
plt.figure(figsize=(10, 7))
for cluster in np.unique(df['Cluster']):
    clustered_data = df[df['Cluster'] == cluster]
    label = 'Outlier' if cluster == -1 else f'Cluster {cluster}'
    plt.scatter(clustered_data['Sensor_1'], clustered_data['Sensor_2'], label=label, s=50)
plt.title('DBSCAN Clustering and Outlier Detection')
plt.xlabel('Sensor 1')
plt.ylabel('Sensor 2')
plt.legend()
plt.grid()
plt.show()

# 4. 異常値のリスト
outliers = df[df['Cluster'] == -1]
print("異常値のデータ:")
print(outliers)
