import pandas as pd
import numpy as np
from scipy.cluster.hierarchy import dendrogram, linkage, fcluster
import matplotlib.pyplot as plt

# 1. ダミー遺伝子発現データの作成
np.random.seed(42)
genes = [f"Gene_{i}" for i in range(1, 51)]  # 50遺伝子
samples = [f"Sample_{i}" for i in range(1, 21)]  # 20サンプル

data = np.random.rand(20, 50) * 100  # 遺伝子発現量データ（ランダム値）
df = pd.DataFrame(data, index=samples, columns=genes)

# 2. 階層型クラスタリングの適用
linked = linkage(df.T, method='ward')  # 遺伝子間でクラスタリング

# 3. デンドログラムのプロット
plt.figure(figsize=(12, 8))
dendrogram(linked, labels=genes, leaf_rotation=90, leaf_font_size=10)
plt.title('Dendrogram of Gene Expression Data')
plt.xlabel('Genes')
plt.ylabel('Euclidean Distances')
plt.grid(True)
plt.show()

# 4. クラスタの分割
clusters = fcluster(linked, t=4, criterion='maxclust')  # 4つのクラスタに分割
clustered_genes = pd.DataFrame({'Gene': genes, 'Cluster': clusters})

# 5. クラスタごとの遺伝子リスト
print("クラスタごとの遺伝子:")
for cluster in sorted(clustered_genes['Cluster'].unique()):
    genes_in_cluster = clustered_genes[clustered_genes['Cluster'] == cluster]['Gene'].tolist()
    print(f"Cluster {cluster}: {genes_in_cluster}")
